@@ -8,8 +8,22 @@ Released under the Python Software Foundation License.
 This setup helper module automates much of the boilerplate.
 The entry point is the setup_main function, which expects to be
 called with a dictionary mapping variable names to values; the
-variable names supported are as follows (most of them map in
-the obvious way to setup keyword arguments):
+supported variable names are listed below.
+
+The recommended way to call setup_main is to first define all the
+necessary variables as module globals in your setup.py file, and
+then simply do:
+
+    from SetupHelper import setup_main
+    setup_main(globals())
+
+This will pass your module globals to SetupHelper, which will then
+do all the grunt work! (SetupHelper automatically filters out globals
+like __name__, __file__, and so on that are supplied by Python, so they
+won't interfere with yours.)
+
+The variable names supported are as follows (most of them map in a
+fairly obvious way to setup keyword arguments):
 
 __progname__           -- name of the program
 __version__            -- program version (TODO: add __dev_build__)
@@ -191,21 +205,10 @@ __post_install__       -- list of script names to be run after install
                           setup command is anything other than install);
                           each name must be a valid command at a shell prompt
 
-Note that this list of variables is for metadata version 1.0 (PEP 241);
-this module does not (yet) support the additional metadata defined in
-version 1.1 (PEP 314), except for the (very limited) support for the
-requires keyword (which is not included in the metadata that are
-registered with PyPI) unless you are using setuptools.
-
-The recommended way to call setup_main is to first define all the
-necessary variables as module globals in your setup.py file, and
-then simply do:
-
-    from SetupHelper import setup_main
-    setup_main(globals())
-
-This will pass your module globals to SetupHelper, which will then
-do all the grunt work!
+Note that this list of variables is for metadata version 1.1 (PEP 314);
+SetupHelper supports that version even if your version of Python does
+not (2.4 and earlier). Note that this applies whether or not you are
+using setuptools; setuptools uses the distutils metadata support.
 
 The setup_main function also allows you to override several of the
 defaults for the SetupHelper class; the currently accepted overrides
@@ -243,7 +246,7 @@ script's globals as described above.
 import sys
 import os
 
-SETUPHELPER_VERSION = (0, 3)
+SETUPHELPER_VERSION = (0, 4)
 
 def version_tuple(ver_s, dev=False):
     """
@@ -289,8 +292,7 @@ def version_string(ver_t, dev_t=None):
     """
     
     try:
-        result = map(int, ver_t)
-        result = '.'.join(map(str, result))
+        result = '.'.join(map(str, map(int, ver_t)))
         if dev_t is not None:
             result += dev_string(dev_t)
     except ValueError:
@@ -365,16 +367,6 @@ if sys.version < '2.5':
     
     class DistributionMetadata(_DistributionMetadata):
         
-        _METHOD_BASENAMES = _DistributionMetadata._METHOD_BASENAMES
-        
-        if sys.version < '2.2.3':
-            _METHOD_BASENAMES = _METHOD_BASENAMES + (
-                "classifiers", "download_url" )
-        
-        _METHOD_BASENAMES = _METHOD_BASENAMES + (
-                # PEP 314
-                "provides", "requires", "obsoletes" )
-        
         def __init__ (self):
             _DistributionMetadata.__init__(self)
             
@@ -386,9 +378,6 @@ if sys.version < '2.5':
             self.provides = None
             self.requires = None
             self.obsoletes = None
-            
-            # Hack to catch setuptools' different keyword
-            self.install_requires = None
         
         def _write_item(self, file, name, value):
             file.write('%s: %s\n' % (name, value))
@@ -439,8 +428,7 @@ if sys.version < '2.5':
         # PEP 314, abbreviated from 2.6 (no set_ methods)
         
         def get_requires(self):
-            # Includes setuptools hack
-            return self.install_requires or self.requires or []
+            return self.requires or []
         
         def get_provides(self):
             return self.provides or []
@@ -465,6 +453,7 @@ def distutils_default():
         # requires variable/keyword.
         """
         
+        # TODO: add a command line option to specify a different repository base URL
         pypi_url = "http://pypi.python.org/simple/"
         user_agent = "Python-urllib/%s SetupHelper/%s" % (
             urllib2.__version__, SETUPHELPER_VERSION)
@@ -601,7 +590,7 @@ def distutils_default():
                 return versions[0]
             return None
         
-        def get_latest_version(self, data, progname, hrefs=False):
+        def get_latest_version(self, data, progname, spec, hrefs=False):
             """
             Return the tuple for the latest version of progname, with the most
             preferred archive type if more than one is available of that version.
@@ -609,6 +598,7 @@ def distutils_default():
             message and return None.
             """
             
+            # TODO: incorporate spec into logic
             versions = self.find_archives(data, progname, hrefs)
             return self.latest_sorted_version(versions)
         
@@ -633,7 +623,8 @@ def distutils_default():
                 return self.latest_sorted_version(versions)
             return None
         
-        def get_latest_local_version(self, req, clean=False):
+        def get_latest_local_version(self, req, spec, clean=False):
+            # TODO: incorporate spec into logic
             result = None
             if self.pypi_local and not clean:
                 # Check for a local repository; we assume that pypi_local
@@ -646,10 +637,10 @@ def distutils_default():
                             self.srcdir = srcdir
             return result
         
-        def process_req(self, req, latest_local, link_data=None, clean=False):
+        def process_req(self, req, spec, latest_local, link_data=None, clean=False):
             if (link_data is not None) and (not clean):
                 # Find latest online version and url by looking in link_data
-                latest = self.get_latest_version(link_data, req)
+                latest = self.get_latest_version(link_data, req, spec)
             else:
                 latest = None
             
@@ -757,6 +748,7 @@ def distutils_default():
                 # Now process the requirements
                 for req in self._requires:
                     # TODO: Strip version specifier off end and store
+                    spec = None
                     
                     # First try to import it -- assumes req is an importable name
                     # (we don't do this if cleaning because we may want to clean
@@ -781,29 +773,50 @@ def distutils_default():
                                 self._req_data[req] = link_data
                     
                     # Check local repository if there is one
-                    latest_local = self.get_latest_local_version(req, clean)
+                    latest_local = self.get_latest_local_version(req, spec, clean)
                     
                     if (link_data is not None) or (latest_local is not None):
                         print "Processing requirement %s..." % req
-                        self.process_req(req, latest_local, link_data, clean)
+                        self.process_req(req, spec, latest_local, link_data, clean)
                         continue
                     
                     # If we get here, requirement was not found
                     print >>sys.stderr, \
                         "Could not find requirement %s; program may not run." % req
     
-    return (RequiresHelper, Extension, setup)
+    def setup_wrapper(**kwargs):
+        """ Wrapper for distutils setup function. """
+        if 'name' in kwargs:
+            setup(**kwargs)
+    
+    return (RequiresHelper, Extension, setup_wrapper)
 
 def distutils_setuptools():
     """ Support function for setuptools. """
     
-    from ez_setup import use_setuptools
-    use_setuptools()
+    # Make use of ez_setup optional -- if it's not present, assume that
+    # the packager has ensured setuptools support by some other method
+    try:
+        from ez_setup import use_setuptools
+        use_setuptools()
+    except ImportError:
+        pass
     
     from setuptools import setup
     from setuptools.extension import Extension
     
-    return (None, Extension, setup)
+    def setup_wrapper(**kwargs):
+        """ Wrapper for setuptools setup function. """
+        if 'name' in kwargs:
+            setup(**kwargs)
+        elif 'install_requires' in kwargs:
+            from setuptools.command.easy_install import main
+            sys.argv[0] = "easy_install"
+            opts = sys.argv[1:]
+            for req in kwargs['install_requires']:
+                main([req] + opts)
+    
+    return (None, Extension, setup_wrapper)
 
 # This will be used to process the __license__ variable below
 
@@ -830,14 +843,29 @@ Development Status :: 2 - Pre-Alpha
 class SetupHelper(object):
     """ Helper class to manage setup process. """
     
+    setup_vars = [
+        'setup_main',
+        'SetupHelper',
+        '__setup_func__',
+        '__setup_args__',
+        '__post_install__' ]
+    
+    python_builtins = [
+        '__builtins__',
+        '__doc__',
+        '__file__',
+        '__name__',
+        '__package__' ]
+    
     overrides = [
         'distutils_pkg',
-        'ext_class',
-        'non_modules',
-        'requires_class',
-        'setup_func',
         'url_types' ]
     
+    metavars = [
+        'ext_class',
+        'requires_class',
+        'setup_func' ]
+    
     def dir_to_pkg(self, dirname):
         """ Converts relative directory path into dotted package name. """
         return dirname.replace('/', '.')
@@ -861,22 +889,60 @@ class SetupHelper(object):
         # Add variables that might be imported by the setup.py script
         # (because they'll show up in vars) and that we'll generate
         # (or won't erase) but will not want as keyword args to setup
-        internal_vars.extend(['setup_main', 'SetupHelper',
-            '__setup_func__', '__setup_args__', '__post_install__'])
+        internal_vars.extend(self.setup_vars)
         
-        # Get the setup variables passed to us by the setup script
-        for varname, value in vars.iteritems():
-            # Don't change the Python-provided vars
-            if varname not in ('__builtins__', '__doc__', '__file__', '__name__', '__package__'):
-                setattr(self, varname, value)
+        # First transfer variables to our __dict__ from above args
+        self.init_vars(vars, kwargs)
         
-        # Process overrides passed in keyword arguments, and set to None
-        # any variables not otherwise set
-        for varname in self.overrides:
+        # Now process variables from setup.py (we check for a __progname__
+        # first because if it isn't there, we assume we were called from
+        # the setuphelper script and only need to do requires processing
+        if hasattr(self, '__progname__'):
+            self.process_readme()
+            self.process_vars()
+            self.write_manifest()
+            self.process_metadata()
+        
+        # Now do the requires processing
+        self.process_requires()
+        
+        # Now merge the rest of the variables into the setup args; note
+        # that we merge instead of updating so that keyword arguments
+        # passed to the setup script override variables set in its globals
+        for varname in dir(self):
+            if varname not in internal_vars:
+                if varname == '__progname__':
+                    # Special case this one, we can't use __name__ because it is
+                    # a special Python variable for modules already
+                    setup_arg = 'name'
+                else:
+                    # Strip the leading and trailing underscores
+                    setup_arg = varname.strip('_')
+                if setup_arg not in self.__setup_args__:
+                    self.__setup_args__[setup_arg] = getattr(self, varname)
+    
+    def init_vars(self, vars, kwargs):
+        # First remove overrides that won't be used if we're being
+        # called from the setuphelper script
+        overrides = self.overrides[:]
+        metavars = self.metavars[:]
+        if '__progname__' not in vars:
+            overrides.remove('url_types')
+            metavars.remove('ext_class')
+        
+        # Process overrides passed in keyword arguments
+        for varname in (overrides + metavars):
             attrname = '__%s__' % varname
             value = kwargs.pop(varname, None)
-            if (value is not None) or not hasattr(self, attrname):
-                setattr(self, attrname, value)
+            if (value is not None) or (attrname not in vars):
+                vars[attrname] = value
+        
+        # Now get the setup variables passed to us by the setup script
+        # (plus any overrides that were written in above)
+        for varname, value in vars.iteritems():
+            # Don't change the Python-provided vars
+            if varname not in self.python_builtins:
+                setattr(self, varname, value)
         
         # Set defaults by distutils package
         if self.__distutils_pkg__ is None:
@@ -885,7 +951,7 @@ class SetupHelper(object):
             'distutils_%s' % self.__distutils_pkg__)()
         del self.__distutils_pkg__
         # We only set variables that weren't already set above
-        for varname in ('requires_class', 'ext_class', 'setup_func'):
+        for varname in metavars:
             attrname = '__%s__' % varname
             if getattr(self, attrname) is None:
                 setattr(self, attrname, locals()[varname])
@@ -893,7 +959,8 @@ class SetupHelper(object):
         # The keyword args left over are treated as keyword args for the
         # setup function below
         self.__setup_args__ = dict(kwargs)
-        
+    
+    def process_readme(self):
         # crib our long description from the opening section of
         # the README file
         if hasattr(self, '__start_line__') and hasattr(self, '__end_line__'):
@@ -919,9 +986,11 @@ class SetupHelper(object):
                 self.__long_description__ = '\n'.join(lines)
             del self.__start_line__
             del self.__end_line__
-        
+    
+    def process_vars(self):
         # We'll store the lines that need to go into MANIFEST.in here (and add
-        # more below if applicable)
+        # more below if applicable) -- note the special-casing of SetupHelper.py
+        # depending on whether it is explicitly listed in py_modules or packages
         self.__manifest_in__ = []
         if not hasattr(self, '__rootfiles__'):
             if hasattr(self, '__py_modules__') and ('SetupHelper' in self.__py_modules__):
@@ -934,7 +1003,8 @@ class SetupHelper(object):
             else:
                 self.__rootdirs__ = ["SetupHelper"]
         for rootfile in self.__rootfiles__:
-            if os.path.isfile(rootfile):
+            # Include glob patterns without checking for file existence
+            if os.path.isfile(rootfile) or ('*' in rootfile) or ('?' in rootfile):
                 self.__manifest_in__.append("include %s\n" % rootfile)
         for rootdir in self.__rootdirs__:
             if os.path.isdir(rootdir):
@@ -952,7 +1022,7 @@ class SetupHelper(object):
             self.__package_root__ = "."
         
         # Figure out the pure Python modules list
-        if self.__non_modules__ is None:
+        if not hasattr(self, '__non_modules__'):
             self.__non_modules__ = []
         if not hasattr(self, '__py_modules__'):
             self.__py_modules__ = [entry[:-3] for entry in os.listdir(self.__package_root__)
@@ -1073,7 +1143,8 @@ class SetupHelper(object):
         if hasattr(self, '__data_dirs__'):
             del self.__data_dirs__
         del self.__package_dirs__
-        
+    
+    def write_manifest(self):
         # Write MANIFEST.in -- tmp file first so we don't clobber the current
         # one if there's a problem
         manifest_in = open("MANIFEST.in.tmp", 'w')
@@ -1087,7 +1158,8 @@ class SetupHelper(object):
         if os.path.isfile("MANIFEST.in.old"):
             os.remove("MANIFEST.in.old")
         del self.__manifest_in__
-        
+    
+    def process_metadata(self):
         # If version is a tuple, convert to a string (need to do this before
         # the URLs are determined)
         if isinstance(self.__version__, tuple):
@@ -1150,45 +1222,38 @@ class SetupHelper(object):
         
         # Sort classifiers (not necessary, but looks better)
         self.__classifiers__.sort()
-        
+    
+    def process_requires(self):
         # Grab setup options from our sys.argv so we can use the same
         # ones when processing requirements
-        setup_opts = " ".join([opt for opt in sys.argv if opt not in ("python", "setup.py")])
+        setup_opts = " ".join(opt for opt in sys.argv[1:] if opt not in ("python", "setup.py"))
         
-        # Check to see if we need to even process requirements
-        do_requires = False
-        for cmd in ("build", "clean", "install"):
-            if cmd in setup_opts:
-                do_requires = True
-                break
+        if hasattr(self, '__progname__'):
+            # Check to see if we need to even process requirements
+            do_requires = False
+            for cmd in ("build", "clean", "install"):
+                if cmd in setup_opts:
+                    do_requires = True
+                    break
+        else:
+            # If no progname, we're being called by setuphelper, always do requires,
+            # and prefix "install" to setup options
+            setup_opts = "install %s" % setup_opts
+            do_requires = True
         
-        # Process requires keyword if not using setuptools (in setuptools calling
-        # the setup_func takes care of this); this is the last thing we do before
-        # merging arguments and calling setup
-        if self.__requires_class__ is not None:
-            if hasattr(self, '__install_requires__'):
+        if hasattr(self, '__install_requires__'):
+            # We need to copy the variable name here so it will show up in PKG-INFO
+            # (we can't use __requires__ in the setup script because that breaks setuptools)
+            self.__requires__ = self.__install_requires__
+            if self.__requires_class__ is not None:
+                # Process requires keyword if not using setuptools (in setuptools calling
+                # the setup_func takes care of this); this is the last thing we do before
+                # merging arguments and calling setup
                 if do_requires:
                     self.__requires_class__(self.__install_requires__, setup_opts).process_reqs()
-                # We need to change the variable name (we can't use __requires__ in the
-                # actual setup script because that breaks setuptools)
-                self.__requires__ = self.__install_requires__
+                # We can now delete this variable since it's not used in standard distutils
                 del self.__install_requires__
         del self.__requires_class__
-        
-        # Now merge the rest of the variables into the setup args; note
-        # that we merge instead of updating so that keyword arguments
-        # passed to the setup script override variables set in its globals
-        for varname in dir(self):
-            if varname not in internal_vars:
-                if varname == '__progname__':
-                    # Special case this one, we can't use __name__ because it is
-                    # a special Python variable for modules already
-                    setup_arg = 'name'
-                else:
-                    # Strip the leading and trailing underscores
-                    setup_arg = varname.strip('_')
-                if setup_arg not in self.__setup_args__:
-                    self.__setup_args__[setup_arg] = getattr(self, varname)
     
     def do_setup(self):
         """
